#! /usr/bin/python

# Copyright (c) 2011 Stefano Palazzo <stefano.palazzo@gmail.com>
# This work is licensed under the terms of the GNU General Public
# License version 3. If you haven't received a copy of the GPL,
# go to <http://www.gnu.org/licenses/gpl.txt>.

import sys

# Unity imports
from gi.repository import GLib
from gi.repository import GObject
from gi.repository import Gio
from gi.repository import Dee
# FIXME: Some weird bug in Dee or PyGI makes Dee fail unless we probe
#        it *before* we import the Unity module... ?!
_m = dir(Dee.SequenceModel)
from gi.repository import Unity

import askubuntu_bridge
from google_api import google_search


# TODO
#    BIG ISSUES
#  - The google search api is deprecated, move this to the custom search API
#
#  - ---- Make sure to follow the terms of service PRECISELY! ----
#
#
#    Smaller Issues
#  - Show more results? Info on how to do that is here:
#      <http://code.google.com/apis/websearch/docs/reference.html#_intro_fonje>
#  - Get rid of Loading cursor on launcher after opening a link
#  - Hide launcher icon (open issue, should it be there by default?)
#  - Needs Nice icons (Person and Question should stay, monochrome for groups)
#  - Searching 'Everything' is too slow compared to searching 'Questions'
#  - Test this on a slower internet connection!
#  - The lense can take over the home-dash, find out how that works


#
# The primary bus name we grab *must* match what we specify in our .place file
#
BUS_NAME = "net.launchpad.AskUbuntuPlace"

# Group ids. Must match the offset into the group models
GROUP_QUESTIONS = 0
GROUP_TAGS = 1
GROUP_EVERYTHING = 2
GROUP_USERS = 3
GROUP_BADGES = 4

# Section ids. Must match the offset into the section model
SECTION_EVERYTHING = 0
SECTION_QUESTIONS = 1
SECTION_TAGS = 2
SECTION_USERS = 3
SECTION_BADGES = 4


class Daemon (object):

    active_section = 0

    def __init__(self):
        # Safe for the signal handlers, I don't know what any of the things
        # in this method mean. Should have to ask someone.

        self._entry = Unity.PlaceEntryInfo.new(
            "/net/launchpad/askubuntuplace/mainentry")

        #
        # Set up all the datamodels we'll share with the Unity process
        # See https://wiki.ubuntu.com/Unity/Places
        #
        # Terminology:
        #
        #   - "sections" A set of disjoint browsable categories.
        #                Fx. "Books", "Film", and "Music"
        #
        #   - "groups" A set of labels that partition the result set into
        #              user-visible chunks. "Popular Books", "Recent Books"
        #
        sections_model = Dee.SharedModel.new(BUS_NAME + ".SectionsModel")
        sections_model.set_schema("s", "s")
        self._entry.props.sections_model = sections_model

        groups_model = Dee.SharedModel.new(BUS_NAME + ".GroupsModel")
        groups_model.set_schema("s", "s", "s")
        self._entry.props.entry_renderer_info.props.groups_model = groups_model

        global_groups_model = Dee.SharedModel.new(BUS_NAME +
            ".GlobalGroupsModel")
        global_groups_model.set_schema("s", "s", "s")
        self._entry.props.global_renderer_info.props.groups_model = (
            global_groups_model)

        results_model = Dee.SharedModel.new(BUS_NAME + ".ResultsModel")
        results_model.set_schema("s", "s", "u", "s", "s", "s")
        self._entry.props.entry_renderer_info.props.results_model = (
            results_model)

        global_results_model = Dee.SharedModel.new(BUS_NAME +
            ".GlobalResultsModel")
        global_results_model.set_schema("s", "s", "u", "s", "s", "s")
        self._entry.props.global_renderer_info.props.results_model = (
            global_results_model)

        #
        # Populate the sections and groups once we are in sync with Unity
        #
        sections_model.connect("notify::synchronized",
            self._on_sections_synchronized)
        groups_model.connect("notify::synchronized",
            self._on_groups_synchronized)
        global_groups_model.connect("notify::synchronized",
            self._on_global_groups_synchronized)

        #
        # Set up the signals we'll receive when Unity starts to talk to us
        #

        # The 'active-search' property is changed when the users searches
        # within this particular place
        self._entry.connect("notify::active-search", self._on_search_changed)

        # The 'active-global-search' property is changed when the user searches
        # from the Dash aka Home Screen
        self._entry.connect("notify::active-global-search",
            self._on_global_search_changed)

        # Listen for changes to the secion (Everything, Questions, Tags...)
        self._entry.connect("notify::active-section", self._on_section_change)
        #
        # PlaceEntries are housed by PlaceControllers.
        # You may have mutiple entries per controller if you like.
        # The controller *must* have the DBus Object path you specify
        # in your .place file
        #
        self._ctrl = Unity.PlaceController.new("/net/launchpad/askubuntuplace")
        self._ctrl.add_entry(self._entry)
        self._ctrl.export()

    def get_search_string(self):
        search = self._entry.props.active_search
        return search.get_search_string() if search else None

    def get_global_search_string(self):
        search = self._entry.props.active_global_search
        return search.get_search_string() if search else None

    def _on_section_change(self, entry_info, section):
        print "section change"
        self.active_section = self._entry.props.active_section

    def _on_sections_synchronized(self, sections_model, *args):
        # Column0: display name
        # Column1: GIcon in string format
        sections_model.clear()
        sections_model.append("Everything",
            Gio.ThemedIcon.new("help").to_string())
        sections_model.append("Questions",
            Gio.ThemedIcon.new("help").to_string())
        sections_model.append("Tags",
            Gio.ThemedIcon.new("help").to_string())
        sections_model.append("Users",
            Gio.ThemedIcon.new("stock-person").to_string())
        sections_model.append("Badges",
            Gio.ThemedIcon.new("help").to_string())

    def _on_groups_synchronized(self, groups_model, *args):
        # Column0: group renderer
        # Column1: display name
        # Column2: GIcon in string format
        groups_model.clear()

        # !! The row offsets must match the GROUP_* constants !!
        groups_model.append("UnityDefaultRenderer",
            "Questions",
            Gio.ThemedIcon.new("help").to_string())
        groups_model.append("UnityDefaultRenderer",
            "Tags",
            Gio.ThemedIcon.new("tag-new").to_string())
        groups_model.append("UnityDefaultRenderer",
            "Search",
            Gio.ThemedIcon.new("distributor-logo").to_string())
        groups_model.append("UnityDefaultRenderer",
            "Users",
            Gio.ThemedIcon.new("stock-person").to_string())
        groups_model.append("UnityDefaultRenderer",
            "Badges",
            Gio.ThemedIcon.new("distributor-logo").to_string())

    def _on_global_groups_synchronized(self, global_groups_model, *args):
        # Just the same as the normal groups
        self._on_groups_synchronized(global_groups_model)

    def _on_search_changed(self, *args):
        search = self.get_search_string()
        results = self._entry.props.entry_renderer_info.props.results_model
        print "Search changed to: '%s'" % search
        self._update_results_model(search, results)

    def _on_global_search_changed(self, entry, param_spec):
        search = self.get_global_search_string()
        results = self._entry.props.global_renderer_info.props.results_model
        print "Global search changed to: '%s'" % search
        self._update_results_model(search, results)

    def add_default_results(self, search, model):
        icon_hint = Gio.ThemedIcon.new("distributor-logo").to_string()
        model.append("http://askubuntu.com/questions/ask",
            Gio.ThemedIcon.new("distributor-logo").to_string(),
            GROUP_EVERYTHING,
            "text/html", "Ask your own question",
            "Ask a Question")
        if search:
            model.append("http://askubuntu.com/search?q=" + search.strip(),
                Gio.ThemedIcon.new("search").to_string(), GROUP_EVERYTHING,
                "text/html", "Search on AskUbuntu.com",
                "Find '%s' on AskUbuntu.com" % search)

    def add_questions_results(self, search, model):
        icon_hint = Gio.ThemedIcon.new("help").to_string() # Questions Icon
        for url, title in askubuntu_bridge.get_questions(search):
            model.append(url, icon_hint, GROUP_QUESTIONS, 
                "text/html", title, title)

    def add_tags_results(self, search, model):
        icon_hint = Gio.ThemedIcon.new("tag-new").to_string() # Tags Icon
        for url, title in askubuntu_bridge.get_tags(search):
            model.append(url, icon_hint, GROUP_TAGS,
                "text/html", title, title)



    def add_badges_results(self, search, model):
        icon_hint = Gio.ThemedIcon.new("help").to_string() # Badge Icon
        for url, title in askubuntu_bridge.get_badges(search):
            model.append(url, icon_hint, GROUP_BADGES,
                "text/html", title, title)

    def add_users_results(self, search, model):
        icon_hint = Gio.ThemedIcon.new("stock_person").to_string() # User Icon
        for url, title in askubuntu_bridge.get_users(search):
            model.append(url, icon_hint, GROUP_USERS,
                "text/html", title, title)

    def _update_results_model(self, search, model):
        model.clear()
        if self.active_section == SECTION_EVERYTHING:
            self.add_default_results(search, model)
        if search:
            if self.active_section in (SECTION_EVERYTHING, SECTION_QUESTIONS):
                self.add_questions_results(search, model)
            if self.active_section in (SECTION_EVERYTHING, SECTION_TAGS):
                self.add_tags_results(search, model)
            if self.active_section in (SECTION_EVERYTHING, SECTION_BADGES):
                self.add_badges_results(search, model)
            if self.active_section in (SECTION_EVERYTHING, SECTION_USERS):
                self.add_users_results(search, model)


if __name__ == "__main__":
    session_bus_connection = Gio.bus_get_sync(Gio.BusType.SESSION, None)
    session_bus = Gio.DBusProxy.new_sync(session_bus_connection, 0, None,
        'org.freedesktop.DBus', '/org/freedesktop/DBus',
        'org.freedesktop.DBus', None)
    result = session_bus.call_sync('RequestName',
        GLib.Variant("(su)", (BUS_NAME, 0x4)), 0, -1, None)
    result = result.unpack()[0]
    if result != 1:
        print >> sys.stderr, "Failed to own name %s. Bailing out." % BUS_NAME
        raise SystemExit(1)
    daemon = Daemon()
    GObject.MainLoop().run()
